@@ -2,8 +2,9 @@
 # @Author  : llc
 # @Time    : 2021/4/30 10:14
 
-__version__ = '0.0.2'
 
+from .models.file import FileStorage
+from .openapi import APIBlueprint
 from .openapi import OpenAPI
 
 print(r"""
