@@ -1,11 +1,12 @@
 # -*- coding: utf-8 -*-
 # @Author  : llc
 # @Time    : 2021/4/30 14:25
+import inspect
 import os
 from functools import wraps
 from typing import Optional, List, Dict, Union
 
-from flask import Flask, Blueprint, render_template, request
+from flask import Flask, Blueprint, render_template, request, make_response
 from pydantic import ValidationError, BaseModel
 
 from .models import Info, APISpec, Tag, PathItem, Components
@@ -13,10 +14,10 @@ from .models.common import Reference
 from .models.path import RequestBody
 from .models.security import SecurityScheme
 from .utils import _parse_rule, get_func_parameters, parse_path, parse_query, get_operation, \
-    get_responses, parse_header, parse_cookie, parse_form, parse_json
+    get_responses, parse_header, parse_cookie, parse_form, parse_body
 
 
-def _do_wrapper(func, responses, header, cookie, path, query, form, json, validate_resp, **kwargs):
+def _do_wrapper(func, responses, header, cookie, path, query, form, body, validate_resp, **kwargs):
     """
     validate request and response
     :param func: view func
@@ -26,7 +27,7 @@ def _do_wrapper(func, responses, header, cookie, path, query, form, json, valida
     :param path:
     :param query:
     :param form:
-    :param json:
+    :param body:
     :param validate_resp: boolean, is validate response
     :param kwargs: path args
     :return:
@@ -38,41 +39,43 @@ def _do_wrapper(func, responses, header, cookie, path, query, form, json, valida
     kwargs_ = dict()
     try:
         if header:
-            header_ = header.annotation(**request.headers)
+            header_ = header.annotation(**request.headers if request.headers is not None else {})
             kwargs_.update({"header": header_})
         if cookie:
-            cookie_ = cookie.annotation(**request.cookies)
+            cookie_ = cookie.annotation(**request.cookies if request.cookies is not None else {})
             kwargs_.update({"cookie": cookie_})
         if path:
             path_ = path.annotation(**kwargs)
             kwargs_.update({"path": path_})
         if query:
-            query_ = query.annotation(**request.args.to_dict())
+            query_ = query.annotation(**request.args.to_dict() if request.args.to_dict() is not None else {})
             kwargs_.update({"query": query_})
         if form:
             req_form = request.form.to_dict()
-            req_form.update(**request.files.to_dict())
-            form_ = form.annotation(**req_form)
+            req_form.update(**request.files.to_dict() if request.files.to_dict() is not None else {})
+            form_ = form.annotation(**req_form if req_form is not None else {})
             kwargs_.update({"form": form_})
-        if json:
-            json_ = json.annotation(**request.get_json(silent=True))
-            kwargs_.update({"json": json_})
+        if body:
+            body_ = body.annotation(
+                **request.get_json(silent=True) if request.get_json(silent=True) is not None else {})
+            kwargs_.update({"body": body_})
     except ValidationError as e:
-        return e.json(), 422
+        resp = make_response(e.json())
+        resp.headers['Content-Type'] = 'application/json'
+        resp.status_code = 422
+        return resp
     # validate response(only validate 200)
     resp = func(**kwargs_)
     if validate_resp:
         for key, response in responses.items():
             if key != "200":
                 continue
-            assert issubclass(response, BaseModel), "invalid `pedantic.BaseModel`"
-            try:
-                _resp = resp
-                if isinstance(resp, tuple):  # noqa
-                    _resp = resp[0]
-                response(**_resp)
-            except ValidationError as e:
-                return e.json(), 500
+            assert inspect.isclass(response) and \
+                   issubclass(response, BaseModel), f"{response} is invalid `pydantic.BaseModel`"
+            _resp = resp
+            if isinstance(resp, tuple):  # noqa
+                _resp = resp[0]
+            response(**_resp)
 
     return resp
 
@@ -116,7 +119,7 @@ class APIBlueprint(Blueprint):
         path = get_func_parameters(func, 'path')
         query = get_func_parameters(func, 'query')
         form = get_func_parameters(func, 'form')
-        json = get_func_parameters(func, 'json')
+        body = get_func_parameters(func, 'body')
         if header:
             _parameters = parse_header(header)
             parameters.extend(_parameters)
@@ -139,8 +142,8 @@ class APIBlueprint(Blueprint):
                 "content": content,
             })
             operation.requestBody = requestBody
-        if json:
-            content, components_schemas = parse_json(json)
+        if body:
+            content, components_schemas = parse_body(body)
             self.components_schemas.update(**components_schemas)
             requestBody = RequestBody(**{
                 "content": content,
@@ -186,15 +189,15 @@ class APIBlueprint(Blueprint):
             else:
                 self.paths[uri].delete = operation
 
-        return header, cookie, path, query, form, json
+        return header, cookie, path, query, form, body
 
     def get(self, rule, tags: Optional[List[Tag]] = None, responses=None, validate_resp=True, security=None):
         def decorator(func):
-            header, cookie, path, query, form, json = self._do_decorator(rule, func, tags, responses, security)
+            header, cookie, path, query, form, body = self._do_decorator(rule, func, tags, responses, security)
 
             @wraps(func)
             def wrapper(**kwargs):
-                resp = _do_wrapper(func, responses, header, cookie, path, query, form, json, validate_resp, **kwargs)
+                resp = _do_wrapper(func, responses, header, cookie, path, query, form, body, validate_resp, **kwargs)
                 return resp
 
             options = {"methods": ["GET"]}
@@ -206,11 +209,11 @@ class APIBlueprint(Blueprint):
 
     def post(self, rule, tags: Optional[List[Tag]] = None, responses=None, validate_resp=True, security=None):
         def decorator(func):
-            header, cookie, path, query, form, json = self._do_decorator(rule, func, tags, responses, security, "post")
+            header, cookie, path, query, form, body = self._do_decorator(rule, func, tags, responses, security, "post")
 
             @wraps(func)
             def wrapper(**kwargs):
-                resp = _do_wrapper(func, responses, header, cookie, path, query, form, json, validate_resp, **kwargs)
+                resp = _do_wrapper(func, responses, header, cookie, path, query, form, body, validate_resp, **kwargs)
                 return resp
 
             options = {"methods": ["POST"]}
@@ -222,11 +225,11 @@ class APIBlueprint(Blueprint):
 
     def put(self, rule, tags: Optional[List[Tag]] = None, responses=None, validate_resp=True, security=None):
         def decorator(func):
-            header, cookie, path, query, form, json = self._do_decorator(rule, func, tags, responses, security, "put")
+            header, cookie, path, query, form, body = self._do_decorator(rule, func, tags, responses, security, "put")
 
             @wraps(func)
             def wrapper(**kwargs):
-                resp = _do_wrapper(func, responses, header, cookie, path, query, form, json, validate_resp, **kwargs)
+                resp = _do_wrapper(func, responses, header, cookie, path, query, form, body, validate_resp, **kwargs)
                 return resp
 
             options = {"methods": ["PUT"]}
@@ -238,12 +241,12 @@ class APIBlueprint(Blueprint):
 
     def delete(self, rule, tags: Optional[List[Tag]] = None, responses=None, validate_resp=True, security=None):
         def decorator(func):
-            header, cookie, path, query, form, json = self._do_decorator(rule, func, tags, responses, security,
+            header, cookie, path, query, form, body = self._do_decorator(rule, func, tags, responses, security,
                                                                          "delete")
 
             @wraps(func)
             def wrapper(**kwargs):
-                resp = _do_wrapper(func, responses, header, cookie, path, query, form, json, validate_resp, **kwargs)
+                resp = _do_wrapper(func, responses, header, cookie, path, query, form, body, validate_resp, **kwargs)
                 return resp
 
             options = {"methods": ["DELETE"]}
@@ -255,11 +258,11 @@ class APIBlueprint(Blueprint):
 
     def patch(self, rule, tags: Optional[List[Tag]] = None, responses=None, validate_resp=True, security=None):
         def decorator(func):
-            header, cookie, path, query, form, json = self._do_decorator(rule, func, tags, responses, security, "patch")
+            header, cookie, path, query, form, body = self._do_decorator(rule, func, tags, responses, security, "patch")
 
             @wraps(func)
             def wrapper(**kwargs):
-                resp = _do_wrapper(func, responses, header, cookie, path, query, form, json, validate_resp, **kwargs)
+                resp = _do_wrapper(func, responses, header, cookie, path, query, form, body, validate_resp, **kwargs)
                 return resp
 
             options = {"methods": ["PATCH"]}
@@ -374,7 +377,7 @@ class OpenAPI(Flask):
         path = get_func_parameters(func, 'path')
         query = get_func_parameters(func, 'query')
         form = get_func_parameters(func, 'form')
-        json = get_func_parameters(func, 'json')
+        body = get_func_parameters(func, 'body')
         if header:
             _parameters = parse_header(header)
             parameters.extend(_parameters)
@@ -397,8 +400,8 @@ class OpenAPI(Flask):
                 "content": content,
             })
             operation.requestBody = requestBody
-        if json:
-            content, components_schemas = parse_json(json)
+        if body:
+            content, components_schemas = parse_body(body)
             self.components_schemas.update(**components_schemas)
             requestBody = RequestBody(**{
                 "content": content,
@@ -440,15 +443,15 @@ class OpenAPI(Flask):
             else:
                 self.paths[uri].delete = operation
 
-        return header, cookie, path, query, form, json
+        return header, cookie, path, query, form, body
 
     def get(self, rule, tags: Optional[List[Tag]] = None, responses=None, validate_resp=True, security=None):
         def decorator(func):
-            header, cookie, path, query, form, json = self._do_decorator(rule, func, tags, responses, security)
+            header, cookie, path, query, form, body = self._do_decorator(rule, func, tags, responses, security)
 
             @wraps(func)
             def wrapper(**kwargs):
-                resp = _do_wrapper(func, responses, header, cookie, path, query, form, json, validate_resp, **kwargs)
+                resp = _do_wrapper(func, responses, header, cookie, path, query, form, body, validate_resp, **kwargs)
                 return resp
 
             options = {"methods": ["GET"]}
@@ -460,11 +463,11 @@ class OpenAPI(Flask):
 
     def post(self, rule, tags: Optional[List[Tag]] = None, responses=None, validate_resp=True, security=None):
         def decorator(func):
-            header, cookie, path, query, form, json = self._do_decorator(rule, func, tags, responses, security, "post")
+            header, cookie, path, query, form, body = self._do_decorator(rule, func, tags, responses, security, "post")
 
             @wraps(func)
             def wrapper(**kwargs):
-                resp = _do_wrapper(func, responses, header, cookie, path, query, form, json, validate_resp, **kwargs)
+                resp = _do_wrapper(func, responses, header, cookie, path, query, form, body, validate_resp, **kwargs)
                 return resp
 
             options = {"methods": ["POST"]}
@@ -476,11 +479,11 @@ class OpenAPI(Flask):
 
     def put(self, rule, tags: Optional[List[Tag]] = None, responses=None, validate_resp=True, security=None):
         def decorator(func):
-            header, cookie, path, query, form, json = self._do_decorator(rule, func, tags, responses, security, "put")
+            header, cookie, path, query, form, body = self._do_decorator(rule, func, tags, responses, security, "put")
 
             @wraps(func)
             def wrapper(**kwargs):
-                resp = _do_wrapper(func, responses, header, cookie, path, query, form, json, validate_resp, **kwargs)
+                resp = _do_wrapper(func, responses, header, cookie, path, query, form, body, validate_resp, **kwargs)
                 return resp
 
             options = {"methods": ["PUT"]}
@@ -492,12 +495,12 @@ class OpenAPI(Flask):
 
     def delete(self, rule, tags: Optional[List[Tag]] = None, responses=None, validate_resp=True, security=None):
         def decorator(func):
-            header, cookie, path, query, form, json = self._do_decorator(rule, func, tags, responses, security,
+            header, cookie, path, query, form, body = self._do_decorator(rule, func, tags, responses, security,
                                                                          "delete")
 
             @wraps(func)
             def wrapper(**kwargs):
-                resp = _do_wrapper(func, responses, header, cookie, path, query, form, json, validate_resp, **kwargs)
+                resp = _do_wrapper(func, responses, header, cookie, path, query, form, body, validate_resp, **kwargs)
                 return resp
 
             options = {"methods": ["DELETE"]}
@@ -509,11 +512,11 @@ class OpenAPI(Flask):
 
     def patch(self, rule, tags: Optional[List[Tag]] = None, responses=None, validate_resp=True, security=None):
         def decorator(func):
-            header, cookie, path, query, form, json = self._do_decorator(rule, func, tags, responses, security, "patch")
+            header, cookie, path, query, form, body = self._do_decorator(rule, func, tags, responses, security, "patch")
 
             @wraps(func)
             def wrapper(**kwargs):
-                resp = _do_wrapper(func, responses, header, cookie, path, query, form, json, validate_resp, **kwargs)
+                resp = _do_wrapper(func, responses, header, cookie, path, query, form, body, validate_resp, **kwargs)
                 return resp
 
             options = {"methods": ["PATCH"]}
